package org.cakelab.codegen;

import java.io.IOException;
import java.util.ArrayList;

public class JavaDoc implements CodeGen {
	StringBuffer current = new StringBuffer();
	ArrayList<String> lines = new ArrayList<String>();
	
	public JavaDoc(String firstLine) {
		append(firstLine);
	}


	@Override
	public void write(IndentStreamControl pout) throws IOException {
		if (current.length() > 0) {
			nextLine();
		}
		
		if (lines.size() == 0) return;

		
		String lineStart = null;
		String firstLineStart = lines.get(0).trim();
		if (firstLineStart.startsWith("/**")) {
			firstLineStart = "/**";
			lineStart = " *";
		} else if (firstLineStart.startsWith("/*")) {
				firstLineStart = "/*";
				lineStart = " *";
		} else {
			firstLineStart = "//";
			lineStart = "//";
		}
		
		boolean first = true;
		String lastLine = null;
		for (String line : lines) {
			if (first) {
				first = false;
				if (!line.trim().startsWith("/*")) {
					pout.print("/* ");
				}
			} else {
				if (!line.trim().startsWith(lineStart)) pout.print(lineStart + " ");
			}
			pout.print(line);
			lastLine = line;
		}
		
		if (!lastLine.endsWith("*/")) {
			pout.println(" */");
		}
		
	}
	
	
	public JavaDoc append(String s) {
		for (char c : s.toCharArray()) {
			append(c);
		}
		return this;
	}


	public JavaDoc append(char c) {
		current.append(c);
		if (c == '\n') {
			nextLine();
		}
		return this;
	}

	

	public JavaDoc append(Object obj) {
		if (obj instanceof String) append((String)obj);
		else if (obj instanceof Character) append((char)obj);
		else current.append(obj);
		return this;
	}


	public JavaDoc append(boolean b) {
		current.append(b);
		return this;
	}


	public JavaDoc append(int i) {
		current.append(i);
		return this;
	}


	public JavaDoc append(long lng) {
		current.append(lng);
		return this;
	}


	public JavaDoc append(float f) {
		current.append(f);
		return this;
	}


	public JavaDoc append(double d) {
		current.append(d);
		return this;
	}


	private void nextLine() {
		lines.add(current.toString());
		current = new StringBuffer();
	}


	public void appendln(String string) {
		append(string).append('\n');
	}

}
