package org.cakelab.codegen;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;

public class MemberGen extends VariableGen {
	public static final Comparator<MemberGen> COMPERATOR = new Comparator<MemberGen>() {
		@Override
		public int compare(MemberGen o1, MemberGen o2) {
			int result = o1.getSortPosition() - o2.getSortPosition();
			
			if (result == 0) {
				result = String.CASE_INSENSITIVE_ORDER.compare(o1.name, o2.name);
			}
			
			if (result == 0) {
				result = String.CASE_INSENSITIVE_ORDER.compare(o1.getSignature(), o2.getSignature());
			}
			return result;
		}
	};
	
	private JavaDoc javadoc;
	private ArrayList<String> annotations = new ArrayList<String>();

	public MemberGen(String modifiers, Class<?> type, String name) {
		this(modifiers, type.getCanonicalName(), name);
	}
	
	public MemberGen(String modifiers, String type, String name) {
		super(modifiers, type, name, null);
	}

	/** Controls method's position in code of a class file after sorting. 
	 * Should be overridden by derived classes. */
	public int getSortPosition() {
		return Integer.MAX_VALUE;
	}

	public void setJavaDoc(JavaDoc javadoc) {
		this.javadoc = javadoc;
	}

	public void addAnnotation(String annotation) {
		this.annotations.add(annotation);
	}
	
	@Override
	public void write(IndentStreamControl pout) throws IOException {
		if (javadoc != null) {
			javadoc.write(pout);
		}
		
		for (String a : annotations) {
			pout.println(a);
		}
		super.write(pout);
	}

	
	
	
}
