package org.cakelab.codegen;

import java.io.IOException;

public abstract class MethodGen extends MemberGen {

	protected VariableGen[] parameters;

	public MethodGen(ClassGen classgen, String modifiers, Class<?> type, String methodName, VariableGen ... parameters) {
		this(classgen, modifiers, type.getCanonicalName(), methodName, parameters);
	}
	
	public MethodGen(ClassGen classgen, String modifiers, String type, String methodName, VariableGen ... parameters) {
		super(modifiers, type, methodName);
		this.parameters = parameters;
	}
	
	/**
	 * Returns the method's signature.
	 */
	public String getSignature() {
		String signature = modifiers + " " + typeName + " " + name + "(";
		if (hasParameters()) {
			VariableGen param = parameters[0];
			signature += param.getSignature();
			for (int i = 1; i < parameters.length; i++) {
				param = parameters[i];
				signature += ", " + param.getSignature();
			}
		}
		signature += ")";
		return signature;
	}

	/**
	 * Whether this method has parameters or not.
	 * @return
	 */
	public boolean hasParameters() {
		return parameters != null && parameters.length > 0;
	}

	@Override
	public void write(IndentStreamControl pout) throws IOException {
		super.write(pout);
		pout.println("{");
		pout.indent(+1);
		writeMethodBody(pout);
		pout.indent(-1);
		pout.println("}");
	}

	public abstract void writeMethodBody(IndentStreamControl pout) throws IOException ;

	
	
	
}
