package org.cakelab.codegen;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;

import javax.tools.StandardJavaFileManager;

import org.cakelab.codegen.compiler.JavaSourceFromString;
import org.cakelab.nio.generator.Generator;

public class SourceCodeUnit implements Runnable, CodeGen {

	private Generator generator;
	private File outDir;
	private ClassGen classGen;
	private boolean writeSourceFiles;
	private StandardJavaFileManager fileManager;

	
	protected HashMap<String, ImportGen> imports = new HashMap<String, ImportGen>();
	
	
	public SourceCodeUnit(Generator generator, File outDir, boolean writeSourceFiles) {
		this.generator = generator;
		this.outDir = outDir;
		this.writeSourceFiles = writeSourceFiles;
		this.fileManager = generator.getFileManager();
	}

	public void setClassGen(ClassGen classGen) {
		this.classGen = classGen;
	}


	public File getCompilationUnit() {
		String name = classGen.getPackageName().replace('.','/') + '/' + classGen.getClassName() + ".java";
		return new File(outDir, name);
	}

	public void addImport(String imported) {
		if (!imports.containsKey(imported)) {
			imports.put(imported, new ImportGen(imported));
		}
	}
	
	
	@Override
	public void run() {
		try {
			
			classGen.populateBody();
			
			IndentStreamControl pout;
	
			File compilationUnit = getCompilationUnit();
			
			if (writeSourceFiles) {
				
				compilationUnit.getParentFile().mkdirs();
				FileOutputStream out = new FileOutputStream(compilationUnit);
				pout = new IndentStreamControl(out);
				
				write(pout);
	
				generator.addCompilationUnit(fileManager.getJavaFileObjects(compilationUnit).iterator().next());
			} else {
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				pout = new IndentStreamControl(out);
				
				write(pout);
				
				generator.addCompilationUnit(new JavaSourceFromString(compilationUnit.toString(), out.toString()));
			}
			pout.close();
		} catch (Throwable t) {
			throw new Error(t);
		}
	}

	@Override
	public void write(IndentStreamControl pout) throws IOException {
		pout.println("package " + classGen.getPackageName() + ";");
		pout.println();
		pout.println();
		for (ImportGen importgen : imports.values()) {
			importgen.write(pout);
		}
		
		pout.println();
		pout.println();

		classGen.write(pout);
	}

	
	
}
