package org.cakelab.nio.generator.codegen.struct;

import java.io.IOException;

import org.cakelab.codegen.ClassGen;
import org.cakelab.codegen.IndentStreamControl;
import org.cakelab.codegen.VariableGen;
import org.cakelab.nio.generator.codegen.Utils;
import org.cakelab.nio.meta.FieldMeta;

public class ArrayFieldGetterMethodGen extends ArrayFieldMethodGen {

	public ArrayFieldGetterMethodGen(ClassGen classgen, FieldMeta fieldMeta, int numIndexParams, boolean addValueParam) {
		super(classgen, 
				fieldMeta, 
				getAccessedComponentType(fieldMeta.getType(), numIndexParams).getCanonicalName(), 
				"get" + Utils.toCamelCase(fieldMeta.name), 
				numIndexParams, 
				valueParam(fieldMeta, numIndexParams, addValueParam)
		);
		
	}
	
	private static VariableGen valueParam(FieldMeta fieldMeta, int dimension, boolean addValueParam) {
		if (addValueParam) {
			return new VariableGen(getAccessedComponentType(fieldMeta.getType(), dimension), "value");
		} else {
			return null;
		}
	}

	@Override
	public void writeMethodBody(IndentStreamControl pout) throws IOException {

		
		String[] indices = getIndexParamNames();
		

		boolean elementAccess = isElementAccess();
		
		String resultVar = null;
		
		if (hasValueParam()) {
			resultVar = parameters[0].getName();
		} else if (elementAccess && !fieldMeta.isArrayElementReusable()) {
			resultVar = "result";
			pout.println(fieldMeta.getElementType().type.getCanonicalName() + " " + resultVar + ";");
		} else {
			resultVar = null;
		}
		
		if (resultVar != null) {
			boolean singleElemAccess = fieldMeta.getTypeMeta().dimensions == indices.length;
			if (!singleElemAccess) super.writeArrayAssertion(pout, resultVar, indices);
			writeArrayAccess(pout, Access.READ, resultVar, indices);
			pout.println("return " + resultVar + ";");
		} else {
			String initialiser;
			if (elementAccess) {
				initialiser = Utils.generateStructInitialiser(fieldMeta, fieldMeta.getElementType(), fieldMeta.view);
			} else {
				initialiser = Utils.generateArrayInitialiser(fieldMeta, indices.length);
			}
			pout.println("return " + this.name + "(" + initialiser + generateIndexParameterList(indices) + ");");
		}

	}




}
