package org.cakelab.nio;

import org.cakelab.nio.common.Global;

/**
 * Initialisation of subsystems in respect to API 
 * validation behaviour (using assertions or not).
 * 
 * By default, assertions are expected to be 
 * enabled. Subsystems will check if this is the 
 * case. Once development is finished, you can 
 * call <b><code>Init.debug(false)</code></b>
 * to run your application without API validation. 
 * 
 * @author homac
 * @see #debug(boolean)
 */
public class Init {
	
	protected static boolean API_VALIDATION_ON = true;
	
	/** Activates/deactivates validation of API use 
	 * for debugging purposes. Activated debugging 
	 * requires enabled assertions (java -ea). */
	public static void debug(boolean debug) {
		Init.API_VALIDATION_ON = debug;
		Global.init();
	}
	
	public static boolean isAssertionsEnabled() {
		boolean enabled = false;
		assert enabled = true; // line executes only if assertions are enabled
		return enabled;
	}

}
