package org.cakelab.nio.common;

import org.cakelab.nio.Init;

public class Global extends Init {

	public static final boolean DEBUG;

	
	/** Major library version */
	public static int VERSION_MAJOR = 1;
	/** Minor library version */
	public static int VERSION_MINOR = 0;
	/** Patch version number of the library */
	public static int VERSION_PATCH = 0;
	
	/** 
	 * If this attribute exists, the API is version 1.0.x compatible. 
	 * If it does not exist, generated classes have to be updated to 
	 * the respective version. */
	public static boolean VERSION_COMPATIBLE_1_0 = true;
	
	/**
	 * Name of the field which indicates compatibility with the latest version. 
	 * This is used by the build system of the library to check if all version 
	 * specifiers are consistent (or the developer did a mistake).
	 */
	public static final String VERSION_TEST_FIELD = "VERSION_COMPATIBLE_" + VERSION_MAJOR + "_" + VERSION_MINOR;

	/** Full version number as string containing major.minor.patch version numbers. */
	public static String VERSION_STR = Integer.toString(VERSION_MAJOR) + "." + VERSION_MINOR + "." + VERSION_PATCH;
	
	static {
		DEBUG = Init.API_VALIDATION_ON;
		if (DEBUG && !isAssertionsEnabled()) {
			throw new Error("\n\n"
					+ "*******************************************************\n"
					+ "*                                                     *\n"
					+ "*              Please turn on assertions              *\n"
					+ "*                     OR call                         *\n"
					+ "*                                                     *\n"
					+ "*           org.cakelab.nio.Init.debug(false)         *\n"
					+ "*                                                     *\n"
					+ "*    at an earlier point in time (e.g. in main()).    *\n"
					+ "*                                                     *\n"
					+ "*******************************************************");
		}
	}


	public static void info(String msg) {
		System.out.println("info: " + msg);
	}

	public static void warning(String msg) {
		System.err.println("warning: " + msg);
	}
	
	public static void error(String msg) {
		System.err.println("error: " + msg);
	}
	
	public static void init() {
		// intentionally empty
		
		// The call to this method is just 'syntactic sugar' and
		// forces the application to load this class and execute the
		// static initialiser. This allows us to keep the field DEBUG
		// as constant rather then variable.
		
		// This method is called at least once by a static initialiser 
		// in class DataViewBase.
	}


}
