package org.cakelab.nio.meta;

public class MetadataUtils {


	public static boolean isKnownImmutableClass(Class<?> type) {
		if (type == String.class) {
			return true;
		} else {
			return isBoxedPrimitive(type);
		}
	}
	
	public static boolean isBoxedPrimitive(Class<?> type) {
		if (type == Byte.class) {
			return true;
		} else if (type == Short.class) {
			return true;
		} else if (type == Integer.class) {
			return true;
		} else if (type == Long.class) {
			return true;
		} else if (type == Float.class) {
			return true;
		} else if (type == Double.class) {
			return true;
		} else if (type == Character.class) {
			return true;
		} else if (type == Boolean.class) {
			return true;
		} else {
			return false;
		}
	}
	
	public static boolean derivesFrom(Class<?> type, Class<?> superClassOrInterface) {
		if (type == null) {
			return false;
		} else if (type == superClassOrInterface) {
			return true;
		} else {
			Class<?> superClass = type.getSuperclass();
			if (derivesFrom(superClass, superClassOrInterface)) {
				return true;
			}
			for (Class<?> superInterface : type.getInterfaces()) {
				if (derivesFrom(superInterface, superClassOrInterface)) {
					return true;
				}
			}
		}
		return false;
	}

	public static Class<?> getBoxedType(Class<?> primitive) {
		assert (primitive.isPrimitive());
		if (primitive == byte.class) return Byte.class;
		else if (primitive == short.class) return Short.class;
		else if (primitive == int.class) return Integer.class;
		else if (primitive == long.class) return Long.class;
		else if (primitive == float.class) return Float.class;
		else if (primitive == double.class) return Double.class;
		else if (primitive == char.class) return Character.class;
		else if (primitive == boolean.class) return Boolean.class;
		else throw new Error("Unsupported primitive type " + primitive.getCanonicalName());
	}

	public static Class<?> getPrimitiveType(Class<?> boxedType) {
		if (boxedType == Byte.class) return byte.class;
		else if (boxedType == Short.class) return short.class;
		else if (boxedType == Integer.class) return int.class;
		else if (boxedType == Long.class) return long.class;
		else if (boxedType == Float.class) return float.class;
		else if (boxedType == Double.class) return double.class;
		else if (boxedType == Character.class) return char.class;
		else if (boxedType == Boolean.class) return boolean.class;
		else throw new Error("class " + boxedType.getClass().getSimpleName() + " is not a boxed type");
	}
	

	public static int getScalarSize(Class<?> type) {
		if (type.equals(byte.class) || type.equals(boolean.class)) {
			return Byte.BYTES;
		} else if (type.equals(char.class)) {
			return Character.BYTES;
		} else if (type.equals(short.class)) {
			return Short.BYTES;
		} else if (type.equals(int.class)) {
			return Integer.BYTES;
		} else if (type.equals(long.class)) {
			return Long.BYTES;
		} else if (type.equals(float.class)) {
			return Float.BYTES;
		} else if (type.equals(double.class)) {
			return Double.BYTES;
		}
		throw new Error("unknown size of scalar type " + type.getSimpleName());
	}



	public static Class<?> getArrayElementType(Class<?> arrayType) {
		Class<?> elemType = arrayType;
		while(elemType.isArray()) elemType = elemType.getComponentType();
		return elemType;
	}

	public static int getArrayDimensions(Class<?> arrayType) {
		int depth = 0;
		while (arrayType.isArray()) {
			arrayType = arrayType.getComponentType();
			depth++;
		}
		return depth;
	}

	/**
	 * Returns the toplevel class of the compilation unit the given type belongs to. 
	 * In other words, it returns the toplevel enclosing class of a given class.
	 * 
	 * @param type
	 * @return
	 */
	public static Class<?> getToplevelEnclosingClass(Class<?> type) {
		if (type.isMemberClass()) return getToplevelEnclosingClass(type.getEnclosingClass());
		else return type;
	}



	
}
