package org.cakelab.nio.view.array;

import java.lang.reflect.Array;

import org.cakelab.nio.view.GenericDataView;
import org.cakelab.nio.view.ReusableDataView;
import org.cakelab.nio.view.impl.ArrayViewBase;

public class CArrayView<C, CV extends GenericDataView<C>> extends ArrayViewBase<C[], CV> {

	public static <E, EV extends GenericDataView<E>> CArrayView<E, EV> create(EV elemView, int length0) {
		return new CArrayView<E, EV>(elemView, length0);
	}
	
	public static <C, CV extends GenericDataView<C>> CArrayView<C[], CArrayView<C, CV>> create(CV elemView, int length0, int length1) {
		return new CArrayView<C[], CArrayView<C, CV>>(create(elemView, length1), length0);
	}
	
	public static <C, CV extends GenericDataView<C>> CArrayView<C[][], CArrayView<C[], CArrayView<C, CV>>> create(CV elemView, int length0, int length1, int length2) {
		return new CArrayView<C[][], CArrayView<C[], CArrayView<C, CV>>>(create(elemView, length1, length2), length0);
	}
	
	public static <C, CV extends GenericDataView<C>> CArrayView<C[][][], CArrayView<C[][], CArrayView<C[], CArrayView<C, CV>>>> create(CV elemView, int length0, int length1, int length2, int length3) {
		return new CArrayView<C[][][], CArrayView<C[][], CArrayView<C[], CArrayView<C, CV>>>>(create(elemView, length1, length2, length3), length0);
	}
	
	
	public CArrayView(CV componentView, int length) {
		super(componentView, length);
	}

	@Override
	public C[] newDataInstance() {
		@SuppressWarnings("unchecked")
		C[] array = (C[]) Array.newInstance(componentView.javatype(), length);
		if (componentView instanceof ReusableDataView) {
			for (int i = 0; i < length; i++) {
				array[i] = ((ReusableDataView<C>)componentView).newDataInstance();
			}
		}
		return array;
	}

	@Override
	protected C[] getArray(int offset, C[] result) {
		if (componentView instanceof ReusableDataView) {
			for (int i = 0; i < length; i++) {
				componentView.offset(offset);
				((ReusableDataView<C>)componentView).get(result[i]);
				offset += stride;
			}
		} else {
			for (int i = 0; i < length; i++) {
				componentView.offset(offset);
				result[i] = componentView.get();
				offset += stride;
			}
		}
		return result;
	}

	public CV view(int index) {
		componentView.offset(getIndexedOffset(index));
		return componentView;
	}

	public CV view(CV view, int index) {
		view.open(__nio__buffer, getIndexedOffset(index));
		return view;
	}
	
	
	public C get(int index) {
		componentView.offset(getIndexedOffset(index));
		return componentView.get();
	}

	public C get(C result, int index) {
		assert(componentView instanceof ReusableDataView);
		componentView.offset(getIndexedOffset(index));
		return ((ReusableDataView<C>)componentView).get(result);
	}

	private int getIndexedOffset(int index) {
		return __nio__offset + index*stride;
	}

	public void set(C value, int index) {
		componentView.offset(getIndexedOffset(index));
		componentView.set(value);
	}
	
	@Override
	protected void setArray(int offset, C[] value) {
		for (int i = 0; i < length; i++) {
			componentView.offset(offset);
			componentView.set(value[i]);
			offset += stride;
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public CArrayView<C,CV> clone() {
		return clone(new CArrayView<C,CV>((CV)componentView.clone(), length));
	}

	public CArrayView<C, CV> clone(CArrayView<C, CV> view) {
		return super.clone(view);
	}

}
