package org.cakelab.nio.view.impl;

import java.lang.reflect.Array;
import java.nio.ByteBuffer;

import org.cakelab.nio.view.ArrayView;
import org.cakelab.nio.view.GenericDataView;

/**
 * 
 * @author homac
 *
 * @param <A> type of the referenced array (e.g. int[][])
 * @param <CV> type of view to access components of the referenced array (e.g. intView or some struct view)
 */
public abstract class ArrayViewBase<A, CV extends GenericDataView<?>> extends DataViewBase implements ArrayView<A> {

	protected final CV componentView;
	protected final int length;
	protected final int size;
	protected final int stride;

	protected ArrayViewBase(CV componentView, int length) {
		super(componentView.alignment());
		this.componentView = componentView;
		this.length = length;
		this.stride = componentView.stride();
		this.size = stride * (length-1) + componentView.sizeof();
	}

	
	@Override
	public void open(ByteBuffer buffer, int offset) {
		componentView.open(buffer, offset);
		super.open(buffer, offset);
	}
	
	
	
	
	@Override
	public void offset(int offset) {
		componentView.offset(offset);
		super.offset(offset);
	}


	@Override
	public void close() {
		componentView.close();
		super.close();
	}

	@Override
	public void set(A value) {
		setArray(__nio__offset, value);
	}
	
	@Override
	public A get() {
		return get(newDataInstance());
	}

	@Override
	public A get(A array) {
		return (A) getArray(__nio__offset, array);
	}

	
	@Override
	public int length() {
		return length;
	}


	@Override
	public int sizeof() {
		return size;
	}
	
	@Override
	public int componentStride() {
		return stride;
	}
	
	@SuppressWarnings("unchecked")
	public Class<A> javatype() {
		// TODO: try to avoid instantiating an array just to get its type
		return (Class<A>) Array.newInstance(componentView.javatype(),0).getClass();
	}

	public abstract A newDataInstance();
	
	
	protected abstract A getArray(int offset, A result);

	protected abstract void setArray(int offset, A result);


}
