package org.cakelab.nio.view.impl;

import java.nio.ByteBuffer;

import org.cakelab.nio.common.Global;
import org.cakelab.nio.view.DataView;

public abstract class DataViewBase implements DataView {
	
	
	protected ByteBuffer __nio__buffer = null;
	protected int 		 __nio__offset = 0;
	
	/** 
	 * alignment is stored as a bit mask, which equals (given_alignment-1).
	 * @see #alignment()
	 */
	private final int __nio__alignment;

	public DataViewBase(short alignment) {
		assert Integer.bitCount(alignment) == 1 && alignment > 0 : "alignment has to be greater then 0 and a power of two";
		this.__nio__alignment = alignment-1;
	}

	public DataViewBase(DataViewBase that) {
		this.__nio__buffer = that.__nio__buffer;
		this.__nio__offset = that.__nio__offset;
		this.__nio__alignment = that.__nio__alignment;
	}
	
	/**
	 * {@inheritDoc}<br/><br/>
	 * Default implementation calls {@link #open(ByteBuffer, int)} with parameters <code>(buffer, buffer.position())</code>.
	 */
	@Override
	public int open(ByteBuffer buffer) {
		int offset = buffer.position();
		open(buffer, offset);
		return offset;
	}
	
	/**
	 * {@inheritDoc}<br/><br/>
	 * Default implementation sets the attributes __nio__buffer and __nio__offset accordingly.
	 */
	@Override
	public void open(ByteBuffer buffer, int offset) {
		assert(buffer != null);
		assert(offset >= 0);
		
		this.__nio__buffer = buffer;
		this.__nio__offset = offset;
	}
	
	@Override
	public void close() {
		__nio__buffer = null;
	}
	
	
	@Override
	public ByteBuffer buffer() {
		return __nio__buffer;
	}
	
	@Override
	public void offset(int offset) {
		assert(offset >= 0);
		
		__nio__offset = offset;
	}
	
	@Override
	final 
	public int offset() {
		return __nio__offset;
	}
	
	/** 
	 * @return alignment, originally given to this data view.
	 */
	@Override
	final 
	public short alignment() {
		return (short)(__nio__alignment+1);
	}

	@Override
	public int align(int offset) {
		return (offset + __nio__alignment) & ~__nio__alignment;
	}
	
	@Override
	public int stride() {
		return align(sizeof());
	}
	
	
	@Override
	public abstract DataView clone();

	@Override
	public <V extends DataView> V clone(V view) {
		assert(view.getClass() == this.getClass()) : "provided view has to be of the same class '" + this.getClass().getCanonicalName() + "'";
		assert(view.alignment() == this.alignment()) : "provided view has to have the same alignment '" + alignment() + "'";
		view.open(this.__nio__buffer, this.__nio__offset);
		return view;
	}

	static {
		// This is can be anywhere else, but should get called at least once.
		Global.init();
	}
}
