package org.cakelab.nio.view.impl;

import org.cakelab.nio.view.ArrayView;

/**
 * 
 * @author homac
 *
 * @param <A> type of the referenced array (e.g. int[][])
 * @param <CV> type of view to access components of the referenced array (e.g. intView or some struct view)
 */
public abstract class ScalarArrayViewBase<A> extends DataViewBase implements ArrayView<A> {

	protected final int length;
	protected final int size;
	protected final int stride;

	protected ScalarArrayViewBase(Class<A> arraytype, short alignment, int elemSize, int length) {
		super(alignment);
		
		this.length = length;
		this.stride = align(elemSize);
		int algndiff = stride - elemSize;
		
		int size = stride * length;
		// remove alignment of first element
		if (algndiff > 0) size -= (algndiff);
		this.size = size;
	}

	public ScalarArrayViewBase(ScalarArrayViewBase<A> that) {
		super(that);
		this.length = that.length;
		this.size = that.size;
		this.stride = that.stride;
	}

	protected abstract int getElemSize();

	protected int getIndexedOffset(int index) {
		return __nio__offset + index*stride;
	}


	
	@Override
	public void set(A value) {
		setArray(__nio__offset, value);
	}
	
	@Override
	public A get() {
		return get(newDataInstance());
	}

	@Override
	public A get(A array) {
		return (A) getArray(__nio__offset, array);
	}

	
	@Override
	public int length() {
		return length;
	}


	@Override
	public int sizeof() {
		return size;
	}
	
	@Override
	public int componentStride() {
		return stride;
	}
	
	public abstract A newDataInstance();
	
	
	protected abstract A getArray(int offset, A result);

	protected abstract void setArray(int offset, A result);


}
