package org.cakelab.nio.view.impl;

import java.nio.ByteBuffer;

import org.cakelab.nio.view.ArrayView;
import org.cakelab.nio.view.scalar.ScalarView;

/**
 * 
 * @author homac
 *
 * @param <A> type of the referenced array (e.g. int[][])
 * @param <CV> type of view to access components of the referenced array (e.g. intView or some struct view)
 */
public abstract class ScalarViewArrayViewBase<A, EV extends ScalarView> extends ScalarArrayViewBase<A> implements ArrayView<A> {

	protected EV elemView;

	protected ScalarViewArrayViewBase(Class<A> arraytype, EV elemView, int length) {
		super(arraytype, elemView.alignment(), elemView.sizeof(), length);
		this.elemView = elemView;
	}

	public ScalarViewArrayViewBase(ScalarViewArrayViewBase<A,EV> that) {
		super(that);
		this.elemView = that.elemView;
	}

	@Override
	protected int getElemSize() {
		return elemView.sizeof();
	}

	@Override
	public void open(ByteBuffer buffer, int offset) {
		elemView.open(buffer, offset);
		super.open(buffer, offset);
	}

	@SuppressWarnings("unchecked")
	public <V extends ScalarViewArrayViewBase<A,EV>> V clone(V view) {
		view.elemView = (EV) this.elemView.clone();
		return super.clone(view);
	}

	public EV view(int index) {
		elemView.offset(getIndexedOffset(index));
		return elemView;
	}
	
	public EV view(EV view, int index) {
		view.open(__nio__buffer, getIndexedOffset(index));
		return view;
	}
	
	
	
}
