package org.cakelab.nio.view.string;


import java.nio.CharBuffer;

import org.cakelab.nio.view.GenericDataView;
import org.cakelab.nio.view.impl.DataViewBase;


/** 
 * This is the standard view for strings which have no user declared view.
 * 
 * @author homac
 *
 */
public class StringView extends DataViewBase implements GenericDataView<String> {
    
	private final char[] chars;
	private int length;
	
	
	public StringView(int length) {
		this((short)1, length);
	}
	
	public StringView(short alignment, int length) {
		super(alignment);
		this.chars = new char[length];
		this.length = length;
	}
	
    public char[] getChars(char[] result) {
        assert(
            result != null
            && result.length >= length
        );
        int __nio__current_offset = __nio__buffer.position();
        __nio__buffer.position(__nio__offset);
        CharBuffer tmpBuffer = __nio__buffer.asCharBuffer();
        tmpBuffer.get(result, 0, length);
        __nio__buffer.position(__nio__current_offset);
        return result;
    }
    
    public char[] getChars() {
        return getChars(new char[length]);
    }
    
    public void setChars(char[] chars) {
        assert(
            chars != null
            && chars.length <= length
        );
        int __nio__current_offset = __nio__buffer.position();
        __nio__buffer.position(__nio__offset);
        CharBuffer tmpBuffer = __nio__buffer.asCharBuffer();
        tmpBuffer.put(chars);
        if (chars.length < length) tmpBuffer.put('\0');
        __nio__buffer.position(__nio__current_offset);
    }
    
    @Override
    public String get() {
		getChars(chars);
		int length = 0;
		while (length < chars.length && chars[length] != '\0') length++;
		return new String(chars, 0, length);
    }
    
    @Override
    public void set(String value) {
        setChars(value.toCharArray());
    }
    
    @Override
    public int sizeof() {
        return length*2;
    }
    
    @Override
    public StringView clone() {
    	return super.clone(new StringView(this.alignment(), this.length));
    }

	@Override
	public Class<String> javatype() {
		return String.class;
	}
    
    
}
