package test.array.complex;

import java.nio.ByteBuffer;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.array.CArrayView;
import org.cakelab.nio.view.generated.test.array.complex.MyStructArrayAlignedView;
import org.cakelab.nio.view.generated.test.array.complex.MyStructArrayAlignedView.ArrayElemView;

import test.array.complex.MyStructArrayAligned.ArrayElem;

public class TestMyStructArrayAligned extends TestBase {
	public static void test(int alignment) {

		MyStructArrayAlignedView view = new MyStructArrayAlignedView((short)alignment);
		
		System.out.println("testing " + MyStructArrayAlignedView.class.getSimpleName() + " (align=" + alignment + ")");
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(view);
		rng.next();
		test(view);
		view.close();
		
		
		System.out.println("testing stream of " + MyStructArrayAligned.class.getSimpleName() + " instances");

		int length = 3;
		if (alignment == 1) assert view.stride() == view.sizeof();
		ByteBuffer buf2 = ByteBuffer.allocate(view.stride()*(length-1) + view.sizeof());
		DataViewIterator<MyStructArrayAlignedView> stream = DataViewIterator.create(view, buf2);
		int seed = rng.getSeed();
		for (int i = 0; i < length; i++) {
			rng.next();
			test(stream.next());
		}
		stream.close();
		stream.open(buf2, 0);
		rng.setSeed(seed);
		for (int i = 0; i < length; i++) {
			rng.next();
			MyStructArrayAligned data = createData(view);
			view = stream.next();
			assert(equals(view.get(), data));
		}
		
		stream.close();
	}
	
	public static boolean equals(MyStructArrayAligned o1, MyStructArrayAligned o2) {
		return equals(o1.array, o2.array);
	}

	public static void test(MyStructArrayAlignedView view) {
		
		MyStructArrayAligned data = createData(view);
		
		view.set(data);
		
		assert(equals(data.array, view.getArray()));
		
		MyStructArrayAligned test = view.get();
		assert(equals(data.array, test.array));

		clearBuffer(view, data);
		
		view.setArray(data.array);
		assert(equals(data.array, view.getArray()));
		
		clearBuffer(view, data);

		for (int i = 0; i < data.array.length; i++) {
			view.setArray(data.array[i], i);
			assert(equals(view.getArray(i), data.array[i]));
		}
		
		clearBuffer(view, data);
		
		for (int i = 0; i < data.array.length; i++) {
			for (int j = 0; j < data.array[i].length; j++) {
				view.setArray(data.array[i][j], i, j);
				assert(equals(view.getArray(i,j), data.array[i][j]));
			}
		}
		
		assert(equals(data, view.get()));

		
		CArrayView<ArrayElem[], CArrayView<ArrayElem, ArrayElemView>> vi = view.viewArray();
		CArrayView<ArrayElem[], CArrayView<ArrayElem, ArrayElemView>> vi_clone = view.viewArray().clone();
		CArrayView<ArrayElem, ArrayElemView> vj_clone = vi_clone.view(0);
		ArrayElemView elemView = view.viewArray(0,0).clone();
		
		for (int i = 0; i < vi.length(); i++) {
			CArrayView<ArrayElem, ArrayElemView> vj = vi.view(i);
			for (int j = 0; j < vj.length(); j++) {
				assert equals(vj.view(j).get(), data.array[i][j]);
				assert equals(vj.view(elemView, j).get(), data.array[i][j]);
				assert equals(view.viewArray(i,j).get(), data.array[i][j]);
				assert equals(view.viewArray(elemView, i,j).get(), data.array[i][j]);
			}
			vj_clone = vi_clone.view(vj_clone, i);
			for (int j = 0; j < vj.length(); j++) {
				assert equals(vj_clone.view(elemView, j).get(), data.array[i][j]);
			}
		}
		
	}

	private static void clearBuffer(MyStructArrayAlignedView view, MyStructArrayAligned data) {
		view.set(view.newDataInstance());
		assert !equals(data.array, view.getArray());
	}

	public static boolean equals(ArrayElem[] array1, ArrayElem[] array2) {
		if (array1.length != array2.length) return false;
		for (int i = 0; i < array1.length; i++) {
			if (!equals(array1[i], array2[i]))return false;
		}
		return true;
	}

	public static MyStructArrayAligned createData(MyStructArrayAlignedView view) {
		MyStructArrayAligned value = view.newDataInstance();
		value.array[0][0].a = rng.get(1);
		value.array[0][0].b = rng.get(2.0f);
		value.array[0][1].a = rng.get(3);
		value.array[0][1].b = rng.get(4.0f);

		return value;
	}

	public static boolean equals(ArrayElem[][] array, ArrayElem[][] array2) {
		if (array.length != array2.length) return false;
		for (int i = 0; i < array.length; i++) {
			if (!equals(array[i], array2[i])) return false;
		}
		return true;
	}

	public static boolean equals(ArrayElem a1, ArrayElem a2) {
		return (a1.a == a2.a) && (a1.b == a2.b);
	}


}
