package test.array.scalar;

import java.nio.ByteBuffer;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.generated.test.array.scalar.MyArrayStructView;

public class TestMyArrayStruct extends TestBase {


	public static void test(short align) {

		MyArrayStructView view = new MyArrayStructView(align);
		
		System.out.println("testing " + MyArrayStructView.class.getSimpleName() + "(align=" + align + ")");
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(view);
		rng.next();
		test(view);
		view.close();
		
		
		System.out.println("testing stream of " + MyArrayStructView.class.getSimpleName() + " instances");

		int length = 3;
		ByteBuffer buf2 = ByteBuffer.allocate(view.stride()*(length-1) + view.sizeof());
		DataViewIterator<MyArrayStructView> stream = DataViewIterator.create(view, buf2);
		int seed = rng.getSeed();
		for (int i = 0; i < length; i++) {
			rng.next();
			test(stream.next());
		}
		stream.close();
		stream.open(buf2, 0);
		rng.setSeed(seed);
		for (int i = 0; i < length; i++) {
			rng.next();
			MyArrayStruct data = createData(view);
			view = stream.next();
			assert(equals(view.get(), data));
		}
		
		stream.close();
	}
	
	public static void test(MyArrayStructView view) {

		
		MyArrayStruct data = createData(view);
		
		view.set_byte_array(data._byte_array);
		assert(equals(view.get_byte_array(), data._byte_array));
		MyArrayStruct test = view.get();
		assert(view.get_byte_array(0) == data._byte_array[0]);
		assert(equals(test._byte_array, data._byte_array));
		
		
		view.set_short_array(data._short_array);
		assert(equals(view.get_short_array(), data._short_array));
		assert(view.get_short_array(0) == data._short_array[0]);
		assert(view.get_short_array(1) == data._short_array[1]);
		test = view.get();
		assert(equals(test._short_array, data._short_array));
		

		
		view.set_int_array_array(data._int_array_array);
		assert(equals(view.get_int_array_array(), data._int_array_array));
		for (int i = 0; i < data._int_array_array.length; i++) {
			assert(equals(data._int_array_array[i], view.get_int_array_array(i)));
			for (int j = 0; j < data._int_array_array[i].length; j++) {
				assert(data._int_array_array[i][j] == view.get_int_array_array(i,j));
			}
		}
		test = view.get();
		assert(equals(test._int_array_array, data._int_array_array));

		

		view.set_long_array_array_array(data._long_array_array_array);
		assert(equals(view.get_long_array_array_array(), data._long_array_array_array));

		view.set_long_array_array_array(view.newDataInstance()._long_array_array_array);
		
		for (int i = 0; i < data._long_array_array_array.length; i++) {
			for (int j = 0; j < data._long_array_array_array[i].length; j++) {
				for (int k = 0; k < data._long_array_array_array[i][j].length; k++) {
					view.set_long_array_array_array(data._long_array_array_array[i][j][k], i,j,k);
					assert(view.get_long_array_array_array(i,j,k) == data._long_array_array_array[i][j][k]);
				}
				assert(equals(view.get_long_array_array_array(i,j), data._long_array_array_array[i][j]));
			}
			assert(equals(view.get_long_array_array_array(i), data._long_array_array_array[i]));
		}
		view.set_long_array_array_array(data._long_array_array_array[1][2], 1, 2);
		assert(equals(view.get_long_array_array_array(1,2), data._long_array_array_array[1][2]));
		test = view.get();
		assert(equals(test._long_array_array_array, data._long_array_array_array));
		
		
		
		for (int i = 0; i < data._char_array.length; i++) {
			view.set_char_array(data._char_array[i], i);
			assert(data._char_array[i] == view.get_char_array(i));
		}
		test = view.get();
		assert(equals(test._char_array, data._char_array));
		
		view.set_char_array(view.newDataInstance()._char_array);
		view.set_char_array(data._char_array);
		assert(equals(view.get_char_array(), data._char_array));
		
		view.set_float_array(data._float_array);
		assert(equals(view.get_float_array(), data._float_array));

		// reset
		view.set_float_array(view.newDataInstance()._float_array);
		
		for (int i = 0; i < data._float_array.length; i++) {
			view.set_float_array(data._float_array[i], i);
			assert(data._float_array[i] == view.get_float_array(i));
		}
		test = view.get();
		assert(equals(test._float_array, data._float_array));
		
		view.setDouble_array(data.Double_array);
		assert(equals(view.getDouble_array(), data.Double_array));
		view.setDouble_array(data.Double_array[2], 2);
		test = view.get();
		assert(equals(test.Double_array, data.Double_array));
		assert(equals(test, data));
		
	}

	private static MyArrayStruct createData(MyArrayStructView view) {
		MyArrayStruct data = view.newDataInstance();
		data._byte_array[0] = rng.get((byte)1);
		data._short_array = rng.get(new short[]{1,2});
		data._int_array_array = rng.get(new int[][] { {1,2,3,4,5,6,7,8}, {8,7,6,5,4,3,2,1}});
		data._long_array_array_array = rng.get(new long[][][] {{{1,8},{2,7},{3,6}},{{4,5},{5,4},{6,3}},{{7,2},{8,1},{9,0}},{{10,-1},{11,-2},{12,-3}}});
		data._char_array = rng.get(new char[]{'b','c','d'});
		data._float_array = rng.get(new float[]{ 1f,2f,3f,4f});
		data.Double_array = rng.get(new double[]{ 4f,3f,2f,1f,0});
		return data;
	}

	private static boolean equals(MyArrayStruct test, MyArrayStruct data) {
		return equals(test._byte_array, data._byte_array)
				&& equals(test._short_array, data._short_array)
				&& equals(test._int_array_array, data._int_array_array)
				&& equals(test._long_array_array_array, data._long_array_array_array)
				&& equals(test._char_array, data._char_array)
				&& equals(test._float_array, data._float_array)
				&& equals(test.Double_array, data.Double_array);
	}

}
