package test.bool;

import java.nio.ByteBuffer;
import java.util.Arrays;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.generated.test.bool.MyBoolStructView;

public class TestMyBoolStruct extends TestBase {
	
	public static void test(int alignment) {
		System.out.println("testing view on " + MyBoolStruct.class.getSimpleName() + "(align=" + alignment + ")");
		
		MyBoolStructView view = new MyBoolStructView((short)alignment);
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(0, view);
		test(1, view);
		view.close();

		System.out.println("testing stream of " + MyBoolStruct.class.getSimpleName() + " instances");
		
		int length = 3;
		ByteBuffer buf2 = ByteBuffer.allocate(view.align(view.sizeof())*length);
		DataViewIterator<MyBoolStructView> stream = DataViewIterator.create(view, buf2);
		for (int i = 0; i < length; i++) {
			test(i, stream.next());
		}
		stream.close();
		
	}
	
	public static void test(int seed, MyBoolStructView view) {
		rng.setSeed(seed);
		
		MyBoolStruct data = view.get();
		assert(data.bool == view.getBool());
		
		data.bool = rng.get(true);
		view.setBool(data.bool);
		assert(data.bool == view.getBool());

		assert(Arrays.equals(data.bool_array, view.getBool_array()));

		MyBoolStruct test = view.get();
		assert(data.bool == test.bool);

		// false, true, false, true ..
		data.bool_array[0] = rng.get(false);
		for (int i = 1; i < data.bool_array.length; i++) {
			data.bool_array[i] = !data.bool_array[i-1];
			view.setBool_array(data.bool_array[i], i);
			assert(view.getBool_array(i) == data.bool_array[i]);
		}
		
		view.setBool_array(data.bool_array);
		assert(Arrays.equals(data.bool_array, view.getBool_array()));

		data.bool = !data.bool;
		// true, false, true ..
		for (int i = 0; i < data.bool_array.length; i++) {
			data.bool_array[i] = !data.bool_array[i];
		}

		view.set(data);
		assert(data.bool == view.getBool());
		assert(Arrays.equals(data.bool_array, view.getBool_array()));
		
		view.get(test);
		assert(data.bool == test.bool);
		assert(Arrays.equals(data.bool_array, test.bool_array));
		assert(equals(test, data));
		

		data.bool_array_array = new boolean[][]{{false, true, true},{true, false, false}};
		view.setBool_array_array(data.bool_array_array);
		assert(Arrays.deepEquals(data.bool_array_array, view.getBool_array_array()));
		for (int i = 0; i < data.bool_array_array.length; i++) {
			assert(Arrays.equals(data.bool_array_array[i], view.getBool_array_array(i)));
			for (int j = 0; j < data.bool_array_array[i].length; j++) {
				assert(data.bool_array_array[i][j] == view.getBool_array_array(i,j));
				data.bool_array_array[i][j] = rng.get((i*j)%2 == 0);
				view.setBool_array_array(data.bool_array_array[i][j], i,j);
				assert(data.bool_array_array[i][j] == view.getBool_array_array(i,j));
			}
			assert(Arrays.equals(data.bool_array_array[i], view.getBool_array_array(i)));
		}
		assert(Arrays.deepEquals(data.bool_array_array, view.getBool_array_array()));
		view.get(test);
		assert(equals(test, data));
		
		data.int_boolean = true;
		view.setInt_boolean(data.int_boolean);
		assert(data.int_boolean == view.getInt_boolean());
		
		data.int_boolean = false;
		view.setInt_boolean(data.int_boolean);
		assert(data.int_boolean == view.getInt_boolean());

		data.short_boolean = false;
		view.setShort_boolean(data.short_boolean);
		assert(data.short_boolean == view.getShort_boolean());
		
		data.short_boolean = true;
		view.setShort_boolean(data.short_boolean);
		assert(data.short_boolean == view.getShort_boolean());
		
		data.int_boolean_array = new Boolean[]{rng.get(false), rng.get(true)};
		view.setInt_boolean_array(data.int_boolean_array);
		assert(Arrays.equals(data.int_boolean_array, view.getInt_boolean_array()));
		
	}

	private static boolean equals(MyBoolStruct o1, MyBoolStruct obj) {
		if (o1 == obj)
			return true;
		if (obj == null)
			return false;
		if (o1.getClass() != obj.getClass())
			return false;
		MyBoolStruct other = (MyBoolStruct) obj;
		if (o1.bool != other.bool)
			return false;
		if (!Arrays.equals(o1.bool_array, other.bool_array))
			return false;
		if (!Arrays.deepEquals(o1.bool_array_array, other.bool_array_array))
			return false;
		if (o1.int_boolean == null) {
			if (other.int_boolean != null)
				return false;
		} else if (!o1.int_boolean.equals(other.int_boolean))
			return false;
		if (!Arrays.equals(o1.int_boolean_array, other.int_boolean_array))
			return false;
		if (o1.short_boolean != obj.short_boolean) 
			return false;
		return true;
	}


}
