package test.simple;

import java.nio.ByteBuffer;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.generated.test.simple.MySimpleStructView;



public class TestMySimpleStruct extends TestBase {

	public static void test(int alignment)  {
		MySimpleStructView view = new MySimpleStructView((short)alignment);
		System.out.println("testing " + MySimpleStructView.class.getSimpleName() + " (alignment=" + alignment + ")");
		
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(view);
		rng.next();
		test(view);
		view.close();

		System.out.println("testing stream of " + MySimpleStruct.class.getSimpleName() + " instances");
		
		int length = 3;
		int seed = rng.getSeed();
		ByteBuffer buf2 = ByteBuffer.allocate(view.align(view.sizeof())*length);
		DataViewIterator<MySimpleStructView> stream = new DataViewIterator<MySimpleStructView>(view, buf2);
		for (int i = 0; i < length; i++) {
			rng.next();
			test(stream.next());
		}
		
		// reset
		stream.open(buf2,0);
		rng.setSeed(seed);
		// compare all instances
		for (int i = 0; i < length; i++) {
			rng.next();
			assert(equals(createData(),stream.next().get()));
		}
		stream.close();
		
	}
	
	public static void test(MySimpleStructView view) {
		MySimpleStruct data = createData();
		
		view.set_byte(data._byte);
		assert(view.get_byte() == data._byte);
		
		view.set_char(data._char);
		assert(view.get_char() == data._char);

		view.set_double(data._double);
		assert(view.get_double() == data._double);
		
		view.set_float(data._float);
		assert(view.get_float() == data._float);
		
		view.set_int(data._int);
		assert(view.get_int() == data._int);
		
		view.set_long(data._long);
		assert(view.get_long() == data._long);
		
		view.set_short(data._short);
		assert(view.get_short() == data._short);
		
		assert(equals(data, view.get()));

		// fill with zeros
		view.set(new MySimpleStruct());

		view.set(data);
		assert(view.get_byte() == data._byte);
		assert(view.get_char() == data._char);
		assert(view.get_double() == data._double);
		assert(view.get_float() == data._float);
		assert(view.get_int() == data._int);
		assert(view.get_long() == data._long);
		assert(view.get_short() == data._short);
		
	}
	
	private static MySimpleStruct createData() {
		MySimpleStruct data = new MySimpleStruct();
		data._byte = rng.get((byte)2);
		data._char = rng.get('C');
		data._double = rng.get(4.0);
		data._float = rng.get(5.0f);
		data._int = rng.get(6);
		data._long = rng.get(7L);
		data._short = rng.get((short)8);
		return data;
	}

	public static boolean equals(MySimpleStruct o1, MySimpleStruct other) {
		if (o1._byte != other._byte)
			return false;
		if (o1._char != other._char)
			return false;
		if (Double.doubleToLongBits(o1._double) != Double.doubleToLongBits(other._double))
			return false;
		if (Float.floatToIntBits(o1._float) != Float.floatToIntBits(other._float))
			return false;
		if (o1._int != other._int)
			return false;
		if (o1._long != other._long)
			return false;
		if (o1._short != other._short)
			return false;
		return true;
	}
	


}
