package test.strings;


import java.nio.charset.StandardCharsets;

import org.cakelab.nio.view.impl.StructViewBase;


/** 
 * This is an example of a user defined struct view.
 * 
 * @author homac
 *
 */
public class AsciiStringView extends StructViewBase<String> {
    
	private final byte[] chars;
	private int length;
	
	public AsciiStringView(short alignment, int length) {
		super(alignment);
		this.chars = new byte[length];
		this.length = length;
	}
	
    public byte[] getBytes(byte[] result) {
        assert(
            result != null
            && result.length >= length
        );
        __nio__buffer.position(__nio__offset);
        __nio__buffer.get(result, 0, length);
        return result;
    }
    
    public byte[] getBytes() {
        return getBytes(new byte[length]);
    }
    
    public void setBytes(byte[] chars) {
        assert(
            chars != null
            && chars.length <= length
        );
        __nio__buffer.position(__nio__offset);
        __nio__buffer.put(chars);
        if (chars.length < length) __nio__buffer.put((byte)0);
    }
    
    @Override
    public String get() {
		getBytes(chars);
		int length = 0;
		while (length < chars.length && chars[length] != 0) length++;
		return new String(chars, 0, length, StandardCharsets.US_ASCII);
    }
    
    @Override
    public void set(String value) {
        setBytes(value.getBytes(StandardCharsets.US_ASCII));
    }
    
    @Override
    public int sizeof() {
        return length;
    }
    
    @Override
    public AsciiStringView clone() {
    	return super.clone(new AsciiStringView(this.alignment(), this.length));
    }

	@Override
	public Class<String> javatype() {
		return String.class;
	}
    
    
}
