package test.strings;

import java.nio.ByteBuffer;
import java.util.Arrays;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.generated.test.strings.MyStringStructAlignedView;

public class TestMyStringStructAligned extends TestBase {
	
	public static void test(short align) {
		MyStringStructAlignedView view = new MyStringStructAlignedView(align);
		System.out.println("testing " + MyStringStructAlignedView.class.getSimpleName() + "(align=" + align + ")");
		
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(view);
		rng.next();
		test(view);
		view.close();
		
		
		System.out.println("testing stream of " + MyStringStructAligned.class.getSimpleName() + " instances");

		int length = 3;
		ByteBuffer buf2 = ByteBuffer.allocate(view.stride()*(length-1) + view.sizeof());
		DataViewIterator<MyStringStructAlignedView> stream = DataViewIterator.create(view, buf2);
		int seed = rng.getSeed();
		for (int i = 0; i < length; i++) {
			rng.next();
			test(stream.next());
		}
		stream.close();
		stream.open(buf2, 0);
		rng.setSeed(seed);
		for (int i = 0; i < length; i++) {
			rng.next();
			MyStringStructAligned data = testData(view);
			view = stream.next();
			assert(equals(view.get(), data));
		}
		
		stream.close();
	}
	
	public static void test(MyStringStructAlignedView view) {

		MyStringStructAligned data = testData(view);

		view.setAscii(data.ascii);
		view.setAsciiArray(data.asciiArray);
		view.setStandardString(data.standardString);
		view.setStandardStringArray(data.standardStringArray);

		assert(data.ascii.equals(view.getAscii()));
		assert(Arrays.equals(data.asciiArray, view.getAsciiArray()));
		assert(data.standardString.equals(view.getStandardString()));
		assert(Arrays.deepEquals(data.standardStringArray, view.getStandardStringArray()));
		
		MyStringStructAligned test = view.get();
		assert(equals(test, data));
	}
	
	private static MyStringStructAligned testData(MyStringStructAlignedView view) {
		MyStringStructAligned data = view.newDataInstance();
		data.ascii = rng.get("horrido");
		data.asciiArray = new String[]{rng.get("one"), rng.get("two"), rng.get("thre"), rng.get("four")};
		
		data.standardString = rng.get("std");
		data.standardStringArray = new String[]{rng.get("one"), rng.get("two")};
		return data;
	}

	private static boolean equals(MyStringStructAligned o1, MyStringStructAligned o2) {
		return (o1.ascii.equals(o2.ascii)) 
				&& Arrays.equals(o1.asciiArray, o2.asciiArray)
				&& o1.standardString.equals(o2.standardString)
				&& Arrays.deepEquals(o1.standardStringArray, o2.standardStringArray);
	}
	

}
