package test.struct;

import java.nio.ByteBuffer;
import java.util.Arrays;

import org.cakelab.nio.view.DataViewIterator;
import org.cakelab.nio.view.generated.test.struct.MyStructStructAlignedView;
import org.joml.Vector3f;
import org.joml.Vector4f;

import test.struct.MyStructStructAligned.DoubleEmbedded;
import test.struct.MyStructStructAligned.DoubleEmbedded.EmbeddedEmbedded;
import test.struct.MyStructStructAligned.EmbeddedReference;
import test.struct.MyStructStructAligned.EmbeddedStatic;
import test.struct.MyStructStructAligned.ReferencedExternal;

public class TestMyStructStructAligned extends Test {

	public static void test(short align) {
		MyStructStructAlignedView view = new MyStructStructAlignedView(align);

		System.out.println("testing " + MyStructStructAlignedView.class.getSimpleName() + " (align=" + align + ")");
		ByteBuffer buffer = ByteBuffer.allocate(view.sizeof());
		view.open(buffer);
		test(view);
		rng.next();
		test(view);
		view.close();
		
		System.out.println("testing stream of " + MyStructStructAligned.class.getSimpleName() + " instances");

		
		int length = 3;
		int seed = rng.getSeed();
		ByteBuffer buf2 = ByteBuffer.allocate(view.stride()*(length-1) + view.sizeof());
		DataViewIterator<MyStructStructAlignedView> stream = DataViewIterator.create(view, buf2);
		for (int i = 0; i < length; i++) {
			rng.next();
			test(stream.next());
		}
		stream.close();
		
		stream.open(buf2,0);
		rng.setSeed(seed);
		for (int i = 0; i < length; i++) {
			rng.next();
			MyStructStructAligned data = createData(view);
			view = stream.next();
			assert(equals(view.get(), data));
		}
		stream.close();

	}
	
	public static void test(MyStructStructAlignedView view) {
		

		MyStructStructAligned data = createData(view);
		view.set(data);

		//
		// Test referenced class
		//
		
		view.setReference(data.reference);
		assert(equals(data.reference,view.getReference()));
		assert(equals(data.reference,view.viewReference().get()));

		MyStructStructAligned test = view.get();
		assert(equals(data.reference,test.reference));
		
		test = view.newDataInstance();
		view.getReference(test.reference);
		assert(equals(data.reference,test.reference));

		view.setReference(view.newDataInstance().reference);
		view.viewReference().set(data.reference);
		assert(equals(data.reference,view.getReference()));
		assert(equals(data.reference,view.viewReference().get()));
		test = view.get();
		assert(equals(data.reference,test.reference));
		assert(equals(data, test));

		//
		// Test static member class
		//

		view.setEmbedded(data.embedded);
		assert(equals(data.embedded, view.getEmbedded(test.embedded)));
		assert(equals(data.embedded, view.viewEmbedded().get(test.embedded)));
		assert(equals(data.embedded, view.getEmbedded()));
		assert(equals(data.embedded, view.viewEmbedded().get()));
		test = view.get();
		assert(equals(data.embedded, test.embedded));
		assert(equals(data,test));
		
		//
		// Test referenced external classes + a user implemented view
		//

		view.setReferencedExternal(data.referencedExternal);
		assert(equals(data.referencedExternal, view.getReferencedExternal(test.referencedExternal)));
		assert(equals(data.referencedExternal, view.getReferencedExternal()));
		assert(equals(data.referencedExternal, view.viewReferencedExternal().get(test.referencedExternal)));
		assert(equals(data.referencedExternal, view.viewReferencedExternal().get()));
		
		test = view.get();
		assert(equals(data.referencedExternal, test.referencedExternal));
		assert(equals(data,test));

		
		//
		// Test member class with reference on another member class in a higher scope
		// 
		view.viewEmbeddedReference().setReference(data.embeddedReference.reference);
		
		assert(equals(data.embeddedReference, view.viewEmbeddedReference().get()));
		assert(equals(data.embeddedReference, view.viewEmbeddedReference().get(test.embeddedReference)));
		assert(equals(data.embeddedReference, view.getEmbeddedReference()));
		assert(equals(data.embeddedReference, view.getEmbeddedReference(test.embeddedReference)));
		
		test = view.get();
		assert(equals(data.embeddedReference, test.embeddedReference));
		assert(equals(data,test));
		
		//
		// Test member class with a member class
		//
		view.setDoubleEmbedded(data.doubleEmbedded);
		assert(equals(data.doubleEmbedded, view.getDoubleEmbedded()));
		assert(equals(data.doubleEmbedded, view.viewDoubleEmbedded().get()));
		
		view.setDoubleEmbedded(view.newDataInstance().doubleEmbedded);
		
		view.viewDoubleEmbedded().set(data.doubleEmbedded);
		assert(equals(data.doubleEmbedded, view.getDoubleEmbedded(test.doubleEmbedded)));
		assert(equals(data.doubleEmbedded, view.viewDoubleEmbedded().get(test.doubleEmbedded)));

		view.setDoubleEmbedded(view.newDataInstance().doubleEmbedded);

		view.viewDoubleEmbedded().viewEmbeddedEmbedded().set(data.doubleEmbedded.embeddedEmbedded);
		assert(equals(view.viewDoubleEmbedded().viewEmbeddedEmbedded().get(), data.doubleEmbedded.embeddedEmbedded));
		assert(equals(view.viewDoubleEmbedded().getEmbeddedEmbedded(), data.doubleEmbedded.embeddedEmbedded));
		
		test = view.get();
		assert(equals(data.doubleEmbedded, test.doubleEmbedded));
		assert(equals(data, test));
		
	}

	private static MyStructStructAligned createData(MyStructStructAlignedView view) {
		MyStructStructAligned data = view.newDataInstance();
		data.reference.a = rng.get(1);
		data.reference.b = rng.get(2);

		data.embedded.rgb[0] = rng.get(255.1f);
		data.embedded.rgb[1] = rng.get(254.2f);
		data.embedded.rgb[2] = rng.get(253.3f);

		data.referencedExternal.position = new Vector3f(rng.get(1f),rng.get(2f),rng.get(3f));
		data.referencedExternal.normal = new Vector4f(rng.get(1f),rng.get(0f),rng.get(0f),1f);
		data.referencedExternal.mv_matrix.setPerspective(rng.get(60),rng.get(1.2f),rng.get(0.001f), rng.get(1000.0f));

		data.embeddedReference.reference.rgb[0] = rng.get(128f);
		data.embeddedReference.reference.rgb[1] = rng.get(127f);
		data.embeddedReference.reference.rgb[2] = rng.get(126f);
		
		data.doubleEmbedded.embeddedEmbedded.c = rng.get(4711);

		return data;
	}

	private static boolean equals(MyStructStructAligned o1, MyStructStructAligned o2) {
		return equals(o1.reference,o2.reference)
				&& equals(o1.embedded,o2.embedded)
				&& equals(o1.referencedExternal, o2.referencedExternal)
				&& equals(o1.embeddedReference, o2.embeddedReference)
				&& equals(o1.doubleEmbedded, o2.doubleEmbedded);
	}

	private static boolean equals(DoubleEmbedded o1, DoubleEmbedded o2) {
		return equals(o1.embeddedEmbedded, o2.embeddedEmbedded);
	}

	private static boolean equals(EmbeddedEmbedded o1, EmbeddedEmbedded o2) {
		return o1.c == o2.c;
	}

	private static boolean equals(EmbeddedReference o1, EmbeddedReference o2) {
		return equals(o1.reference,o2.reference);
	}

	private static boolean equals(ReferencedExternal o1, ReferencedExternal o2) {
		return o1.position.equals(o2.position) 
				&& o1.normal.equals(o2.normal) 
				&& o1.mv_matrix.equals(o2.mv_matrix);
	}

	private static boolean equals(EmbeddedStatic o1, EmbeddedStatic o2) {
		return Arrays.equals(o1.rgb, o2.rgb);
	}

	private static boolean equals(ReferencedClass o1, ReferencedClass o2) {
		return o1.a == o2.a && o1.b == o2.b;
	}

}
